var prop = require('propper');

module.exports = {
  name: 'collection',
  selector: '[data-type=collection][data-path]',
  update: function(el, newValue, path) {
    newValue = newValue || {};

    //force the fields to reload their content in the collection
    if (this.oldData) {
      prop(this.oldData, path, {});
    }

    var sortHash = function(item) {
      if (item.weight !== undefined) {
        return item.weight;
      }
      return item.added;
    };

    var sortedKeys = Object.keys(newValue).sort(function(a, b) {
      var aSortHash = sortHash(newValue[a]);
      var bSortHash = sortHash(newValue[b]);
      if (aSortHash === bSortHash) {
        return 0;
      }
      if (aSortHash < bSortHash) {
        return -1;
      }
      return 1;
    });

    var sortedValues = {};
    sortedKeys.forEach(function(key, index) {
      var value = newValue[key];
      value.index = index;
      sortedValues[key] = value;
    });

    var childTemplate = el.getAttribute('data-childTemplate');
    if (!childTemplate) {
      childTemplate = el.innerHTML;
      el.setAttribute('data-childTemplate', childTemplate);
    }

    var newHTML = Object.keys(sortedValues || {}).map(function(id) {
      return childTemplate.replace(/data-(path|show|hide|href)="([^"]*)"/g, function(match, p, path) {
        return 'data-' + p + '="' + id + (path === '.' ? '' : '/' + path) + '"';
      });
    }).join('');

    try {
      el.innerHTML = newHTML; 
    } catch(e) {
      // IE9 has some elements (like tbody) that are readonly for innerHTML
      var childTemplate = el.getAttribute('data-childTemplate');
      if (childTemplate) {
        el.removeAttribute('data-childTemplate');
      }
      var preHtml = el.outerHTML;
      var startTag = preHtml.substr(0, preHtml.indexOf('>'));
      var endTag = preHtml.substr(preHtml.lastIndexOf('<'));
      var replacementHTML = startTag + newHTML + endTag;
      var temp = el.ownerDocument.createElement('div');
      if (el.tagName === 'TBODY') {
        temp.innerHTML = '<table>' + replacementHTML + '</table>';
        if (childTemplate) {
          temp.firstChild.firstChild.setAttribute('data-childTemplate', childTemplate);
        }
        el.parentNode.replaceChild(temp.firstChild.firstChild, el);
      } else {
        temp.innerHTML = replacementHTML;
        if (childTemplate) {
          temp.firstChild.setAttribute('data-childTemplate', childTemplate);
        }
        el.parentNode.replaceChild(temp.firstChild, el);
      }
    }
  },

  hash: function(obj) {
    if (obj === undefined) {
      return undefined;
    }

    return Object.keys(obj).join();
  }
};
