var prop = require('propper');

module.exports = extractFbPath;

function extractFbPath(el, leafProp) {
  var leafProps;
  if (leafProp) {
    leafProps = [leafProp];
  } else {
    leafProps = ['path', 'show', 'hide', 'href'];
  }

  var path;
  for (var i = 0; i < leafProps.length; i++) {
    path = el.getAttribute('data-' + leafProps[i]);
    if (path) {
      leafProp = leafProps[i];
      break;
    }
  }

  if (path && path.match(/^https:\/\//)) {
    return parse(path);
  }

  var pathParts = extractPathParts(el.parentNode);
  if (path) {
    pathParts.push(path);
  }

  var newPath = collapsePathParts(pathParts);
  el.setAttribute('data-' + leafProp, newPath);

  return parse(newPath);
}

extractFbPath.parse = parse;

function parse(fullPath) {
  var match = /^(.*)\/@(\/.*)$/g.exec(fullPath);

  return {
    full: fullPath,
    root: match[1],
    path: match[2]
  };
}

function extractPathParts(el) {
  var parts = [];
  var part;
  do {
    part = el.getAttribute('data-path');
    if (part) {
      if (part.match(/^https:\/\//)) {
        if (part.match(/@/)) {
          parts.push(part);
        } else {
          parts.push(part.replace(/\/$/, '') + '/@');
        }

        break;
      } else {
        parts.push(part);
      }
    }
    el = el.parentNode;
  } while (el);

  return parts.reverse();
}

function collapsePathParts(parts) {
  var cleanParts = [];

  var first = parts.shift();
  parts.join('/').split('/').forEach(function (part) {
    if (part === '') {
      cleanParts = [];
    } else if (part === '..') {
      cleanParts.pop();
    } else if (part !== '.') {
      cleanParts.push(part);
    }
  });

  return first + '/' + cleanParts.join('/');
}